
/********************************************************/
/*							*/
/*	Utility Routines for MX-80 Bit-Plot Graphics	*/
/*        Upgraded for the Prowriter by Don Denbo       */
/*       Patched to output correct wire address by      */
/*            Dana Cotant of Micro Cornucopia           */
/*                                                      */
/*	Don Brittain		3 January 1983		*/
/*		               14 January 1984   DmC	*/
/********************************************************/

#define HSIZE 400
#define VSIZE 408
#define NO 0
#define YES 1
#define LIST 5
#define ESC 27

char grafbuf[HSIZE][VSIZE/8];		/* MUST be EXTERNAL */
extern int horsize, versize, flag;
extern int pixplot(), unpixplot();


init(h,v)	/* Clears or fills the user area of the graph matrix */
		/* depending on the status of the external var "flag" */
int h,v;

{
    register i, j;
    int fill=0;

    if(flag==NO) fill=255;
    v=(v+7)/8;
    for(j=1; j<v; j++)
	for(i=0; i<h; i++)
	    grafbuf[i][j]=fill;
    if(flag)
	for(i=0; i<h; i++)
	    grafbuf[i][0]=0;
    else			/* Cluge resulting because the graph is */
	{			/* defined bit-wise, but initializing */
	switch(versize%8) {	/* is much quicker byte-wise */
	    case 1 :
		fill=1;
		break;
	    case 2 :
		fill=3;
		break;
	    case 3 :
		fill=7;
		break;
	    case 4 :
	        fill=15;
	        break;
	    case 5 :
		fill=31;
		break;
	    case 6 :
		fill=63;
		break;
	    case 7 :
		fill=127;
		break;
	    case 0 :
		fill=255;	}
	for(i=0; i<h; i++)
	    grafbuf[i][0]=fill;
	}
    flag=YES;		/* functions that use flag always reset it */
}


grafprnt(n)	/* routine to send the graph RAM image to the ProWriter */

int n;

{
    register i, j;
    int v;
    setline(8);
    bdos(LIST,ESC);
    bdos(LIST,'>');
    clmar(n);
    v=(versize+7)/8;
    for(j=0; j<v; j++)   {
         bitmode(horsize);
         for(i=0; i<horsize; i++)
	    bdos(LIST,flip(grafbuf[i][j]));        /* flip() inverts byte */
	clmar(n);	}                         /*  for the ProWriter  */
    bdos(LIST,ESC);                              /*   See routine below */
    bdos(LIST,'<');
    setline(12);
}

border()	/* puts a border around the graph */
{
    register i;
    int rside, top;
    
    rside=horsize-1;
    top=versize-1;
    
    if(flag)
	{
	for(i=0; i<horsize; i++)	{
	    pixplot(i,0);
	    pixplot(i,top);		}
	for(i=0; i<versize; i++)	{
	    pixplot(0,i);
	    pixplot(rside,i);		}
	}   
 else
	{
	for(i=0; i<horsize; i++)	{
	    unpixplot(i,0);
	    unpixplot(i,top);		}
	for(i=0; i<versize; i++)	{
	    unpixplot(0,i);
	    unpixplot(rside,i);		}
	}
    flag=YES;
}


setline(n)	/* routine to set line spacing on the Prowriter */
int n;
{
	int n1,n2;
	n = n*2;
	n1 = n/10 + 48;
	n2 = n%10 + 48;
    bdos(LIST,ESC);
    bdos(LIST,'T');
    bdos(LIST,n1);
    bdos(LIST,n2);
}





bitmode(n)	/* sets n chars of dot graphics on Prowriter */
int n;
{
	int n1,n2,n3;
	n1 = n/100 + 48;
	n2 = (n%100)/10 +48;
	n3 = n%10 +48;
    bdos(LIST,ESC);
    bdos(LIST,'S');
    bdos(LIST,48);
    bdos(LIST,n1);
    bdos(LIST,n2);
    bdos(LIST,n3);
}


crlf(n)		/* printer carriage-return-linefeed */
int n;
{
int i;
for(i=0; i<n; i++)
    {
    bdos(LIST,13);
    bdos(LIST,10);
    }
}


margin(n)	/* printer left-margin */
int n;
{
int i;
for(i=0; i<n; i++)
    bdos(LIST,' ');
}


clmar(n)	/* single CRLF then n-space margin to printer */
int n;
{
    crlf(1);
    margin(n);
}
    

/* The following assembly language routine flips the byte that was   */
/* vertically formatted for the MX80 end over end so that it  will   */
/* address the correct wires on the print head of the ProWriter      */
/* A bit is pulled out of one end of the source register and shoved  */
/* in the opposite end of the destination register. This is repeated */
/* 8 times and the bits are just rotated through until the byte has  */
/* been inverted.                                DmC 14 January 1984 */

 
#asm
	public	flip_
flip_:	lxi	h,2
	dad	sp
	mov	a,m
	lxi	h,0
	mvi	d,0
	mvi	e,9h
agn:	dcr	e
	jz	back
	ral
inner:	mov	d,a
	mov	a,l
	rar
	mov	l,a
	mov	a,d
	jmp	agn
back:	mov	a,l
	ora	a
	ret
#endasm
