
@Heading('BATCHING' FILES)



Often you will wish to print a very long document comprised of a
number of files, such as a book whose various chapters are each
held in a separate file.  When printing this document you will
wish that the files be consecutively numbered, not only by page,
but by chapter and even section.  This would be an awkward thing
to accomplish if each file were printed separately, since Perfect
Writer normally begins page and section numbering anew each time
it prints a file.  However, Perfect Writer provides a means of
appending one or more documents to each other, so that when
formatted and printed, the  documents will be consecutively paged
and sectioned.

@flushleft{@b(The @@INCLUDE Command)}

	If you merely wish to insert one document within another,
the method to use is the "@@Include" command.  For example, at
any point in the current document, type the command:
@center<@b[@@Include(b:thisfile.mss)]>

	When the current file is formatted and printed the file
'thisfile.mss', which is located on drive 'B', will be included
in the current file at the @b(exact point) where the command was
given.  If the included file must begin on its own new line, paragraph,
or page, you will wish to preceed this command with another
command that will accomplish the appropriate spacing.  For
example:

@display<@b[@@newpage
@@Include(b:thisfile.mss)]>




	The "@@Include" command can be repeated  any number of
times within a document.  For example, at the end of the current
document you may wish to append several files:

@display<@B[@@Include(b:thisfile.mss)
@@Include(b:thatfile.mss)
@@Include(b:anyfile.mss)
. . . .etc.]>

	 '@@Style' commands governing the current file will
automatically determine the format of all files which have been
appended using the "@@Include" command.  For example, if the
current document is being double-spaced, "@@Style<spacing 2
lines>", all documents which have been 'included' in the current
document will be double-spaced as well.  

	The single exception to this rule, however, is the
placement of  'Footnotes'.  When  appending one document to
another using the "@@Include" command, footnotes which occur in
the various files @b(must) be placed at the bottom of the page on
which they occur.  If you desire to have footnotes placed on a
page at the end of the document, you must use the following
'batch.mss' method.

@flushleft{@b(Using a 'BATCH FILE') With One Diskette}

	Another way of joining separate files to form one larger
finished document is to create a separate 'batch' file
(i.e. b:batch.mss), which consists of nothing but "@@Include"
commands.  To create such a file, proceed as follows:
@begin(itemize)
From the menu of Perfect Writer select 'E--Edit a file', naming
the file you wish to edit 'b:batch.mss'.

In the new file called 'b:batch.mss' begin listing the various
files to be included:
@display<@b[@@Include(b:thisfile.mss)
@@newpage
@@Include(b:thatfile.mss)
@@newpage
@@Include(b:anyfile.mss)
@@newpage
@@Include(b:everyfile.mss)
etc.]>

Save this file to disk (CTRL--X CTRL--S)
@end(itemize)

	 When the file 'b:batch.mss' is formatted and printed,
each individual file that you list will be included all paged and
sectioned consecutively.  (In the above example, each will start
on its own 'newpage'.)  '@@Style' commands which will govern the
entire document can be placed either in the first file of the
chain, or at the beginning of 'b:batch.mss'.  




@heading(ADVANCED 'BATCH' PROCESSING)
@center(For Two or More Diskettes)

	It may happen that the document you create by batching
files will be too large to fit on a single diskette. That is, 
when formatting a file, Perfect Writer  creates an intermediate
file, called a '.FIN' file, which will be sent to the printer and 
which is itself sometimes as large or larger than the original
file.  If there is not enough room on your diskette to hold both
the original file(s) and the formatted '.FIN' file, you will have
to format and print your document in smaller batches.

	When printing a document in separate batches it is still
possible to have the entire document sequentially paged and
sectioned, using '@@Set' commands to establish correct page and
section numbers (See the discussion of this command form in your
Perfect Writer User's Guide). An example  might be the following:

@center(@b[b:batch2.mss])
@begin(display)
@@set(page=23) *
@@set(chapter=2) *
@@set(section=3
@@include(b:thisfile.mss)
@@include(b:thatfile.mss)
@@include(etc............)
@end(display) 
	
	In this 'second' batch file several '@@Set' commands have been
used to 'set' the page, chapter and section number that will be
used.  The last numbering which occurred in the previous batch
file (i.e. 'b:batch1.mss') was Chapter 3, Section 3, page 23.  In
'b:batch2.mss' the file 'Thisfile.mss' will begin printing on
page '24' and the first section that occurs in that document will
be numbered '3.4' (i.e. Chapter 3, Section 4).  * Notice that a
'set' reference specifies a number @b(one less) than the number
that will actually begin the paging or sectioning.  Thus, if
'b:thisfile.mss' is really a continuation of Chapter 3, the
"@@Set" command must specify Chapter 2.

	The possible pre-defined 'counters' (i.e. 'page', 'chapter',
'section', etc.) are presented in your Perfect Writer User's
Guide.
	

@flushleft{@b(Changing 'END SPACE')}

	When formatting, Perfect Writer divides machine
memory into two parts: that which holds the running text, called
'page space', and that which accumulates footnotes, table of
contents, and index entries, called 'end space'.  If there is
more 'page space' then there is correspondingly less 'end space'
and visa versa.

 If you have a great number of footnotes, and index entries, you
may encounter an error message saying: @example('Internal Error:
Out of Memory.  Error was on line <xx> of file 'b:thisfile.mss'
in end space.')  Should this happen, you will either have to
reduce the number of files in your 'batch file', or increase the
amount of space that has been reserved as 'end space'.  ('Page
space' cannot be increased, except by eliminating 'end-space'.)


	'End space' is initially set at 1000 bytes.  Increasing
it to as much as 8000 bytes will have no noticeable effect,
unless your document is extremely long, in which case you may run
out of 'page space'  (and for which nothing can be done, but
reduce the number of files that are being batched). 


	Use the following procedure to increase  'end space':
@begin(enumerate)
With your computer turned on, insert your Perfect Writer Edit
Diskette in drive 'A' and your Installation diskette in drive 'B'.

Push the 'RESET' button on the computer to initialize both diskettes.

Type:@center(@b[b:pfconfig   <Followed by a RETURN>])  (This
tells the computer to go to drive 'B' and enter the program
'PFCONFIG.COM')

The screen will fill up with information and ask 'Do you wish to
exit and change disks?'  Enter:@center(@b<n  {for 'no'}>)

The screen will fill up again and say 'Type any character to
continue'.  Do this.

'PFCONFIG' will present its main menu.  Choose option number 3, 'Select
Perfect Writer default style parameters'.  Enter:
@center(@b{3
<Followed by a RETURN>})

The screen changes to display the various 'Default Style
Parameters'  (eighteen in all).  Select the last of these, 'End
space', by entering:@center(@b{18   <Followed by a RETURN>})

You are now in a position to change the allocation of
'endspace', which is initially set at 1000 bytes.  Enter the
amount that you desire in 1000 byte increments. (3000 is a good
'endspace' to try.)  Follow the number you enter with a
RETURN and exit the program by typing, in order:
@center(@b{CTRL---G})  (This will return you to the main menu for
'PFCONFIG') then the number@center(@b{'6'   <Followed by a
RETURN.>})   (This will get you out of 'PFCONFIG' and  'Warm
Boot' you to  CP/M, displaying the prompt 'A>'.)
@end(enumerate)

'PFCONFIG' updates the 'PF.DAT' file on your Perfect Writer Edit
Diskette.  Hereafter,  Perfect Writer will observe the new
'endspace' that you have specified.

@center[@b(PERFECT SOFTWARE, INC.)
Product Science Center] parameters'.  Enter:
@center(@b{3
<Followed by a RETURN>})

The screen